
#include "LTC_util.h"


/*************************************/
/*       Variablendefinitionen       */
/*************************************/

union {
  uint16_t	words[5];
  uint8_t	bytes[10];
} sr;

volatile uint16_t oldcapt;
uint8_t  bitcount;

/*************************************/


void USART0_Init(uint16_t bauddiv)
{
  /* Set baud rate */
  UBRR0H = (uint8_t)(bauddiv>>8);
  UBRR0L = (uint8_t)bauddiv;
  /* Enable receiver and transmitter */
  UCSR0B = (1<<RXEN)|(1<<TXEN);
  /* Set frame format: 8data, 1stop bit */
  UCSR0C = (0<<USBS)|(3<<UCSZ0);
}

void usart_putc(uint8_t data)
{
  /* Wait for empty transmit buffer */
  while (!(UCSR0A&(1<<UDRE)));
  /* Put data into buffer, sends the data */
  UDR0 = data;
}

void init_TIMER1(void)
{
  DDRD  &= ~(1<<PD4);		// PD4 (ICP1) = Eingang
  PORTD &= ~(1<<PD4);		// PD4-Pullup disablen
  TCCR1A = 0;
  TCCR1B = (1<<ICNC1)|(1<<CS10);	// Noise Canceler = on ; clk = fcpu
  TIFR   = (1<<ICF1);				// Interruptflag lschen
  TIMSK  = (1<<TICIE1);			// Input Capture Interrupt enablen
}

void addbit (uint8_t bit)
{
  sr.bytes[0] = sr.bytes[0]>>1;
  
  for(uint8_t n=1;n<10;n++)
  {
    if(sr.bytes[n] & 1)
	{ sr.bytes[n-1] |= 0x80;
	}
    sr.bytes[n] = sr.bytes[n]>>1;
  }
  
  if(bit)
  { sr.bytes[9] |= 0x80;
  }
  
  if(bitcount>158)			// erforderliche Anzahl an gltigen Bits erreicht?
  {
#ifdef TC_PARITY
    if(sr.words[4] == 0xBFFC)		// Sync empfangen?
    { uint8_t parity=0;				// Paritt prfen...
      for(uint8_t i=0;i<8;i++)
      {
	    parity += parity_even_bit(sr.bytes[i]);
	  }
	  if(parity&0x01)				// Wenn Paritt gltig:
	  { tc_rdy=1;					// -> Flag setzen
	  }
	}
#else
    if(sr.words[4] == 0xBFFC)		// Sync empfangen?
	{ tc_rdy=1;						// -> Flag setzen
	}
#endif
  }
}

ISR(TIMER1_CAPT_vect)
{
  uint16_t duration = ICR1-oldcapt;	// Pulsdauer bestimmen
  oldcapt=ICR1;
  TOGGLE_CAPT_EDGE;					// Capture-Flanke switchen
  CLEAR_ICF1;						// Interrupt-Flag lschen
  
  bitcount++;		// Zhler erhhen
  
  if(duration<MIN_1)		// Puls war zu kurz
  {
    bitcount=0;				// -> Zhler zurcksetzen
  }
  else if(duration<MAX_1)	// Puls knnte "1" sein
  {
    if(!(bitcount&1))		// zweite kurze Halbwelle?
	{
	  addbit(1);			// -> "1" hinzufgen
	}
  }
  else if(duration<MAX_0)	// Puls ist "0"
  {
    bitcount++;				// Zhler nochmals erhhen
	if(bitcount&1)			// zuvor nur *eine* kurze Halbwelle? (illegal)
	{
	  bitcount=2;			// Zhler zurcksetzen
	}
    addbit(0);				// "0" hinzufgen
  }
  else						// Puls war zu lang
  {
    bitcount=0;				// -> Zhler zurcksetzen
  }
  
  if(bitcount>161)
  { bitcount=160;
  }
  
  if(tc_rdy==1)			// kompletter TC empfangen?
  {							// -> TC kopieren (sichern)
    tc_rdy++;
    tcub[0]=sr.bytes[0];
    tcub[1]=sr.bytes[1];
    tcub[2]=sr.bytes[2];
    tcub[3]=sr.bytes[3];
    tcub[4]=sr.bytes[4];
    tcub[5]=sr.bytes[5];
    tcub[6]=sr.bytes[6];
    tcub[7]=sr.bytes[7];
  }
}
