/*
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/parity.h>
#include <stdint.h>
*/

#include "LTC_util.h"

/*************************************/
/*       Variablendefinitionen       */
/*************************************/

volatile uint8_t  tc[12];	// Timecode
volatile uint8_t  ub[8];		// User Bits

/*************************************/

#define HH_	3	// Stunden
#define MM_	2	// Minuten
#define SS_	1	// Sekunden
#define FF_	0	// Frames

#define FE_	4	// Frames Einer
#define FZ_	5	// Frames Zehner
#define SE_	6	// Sekunden Einer
#define SZ_	7	// Sekunden Zehner
#define ME_	8	// Minuten Einer
#define MZ_	9	// Minuten Zehner
#define HE_	10	// Stunden Einer
#define HZ_	11	// Stunden Zehner

void send_tc (void);
void send_ub (void);

int main (void)
{
  USART0_Init(25);		// Initialisiere USART mit 38400Baud (@16MHz)
  init_TIMER1();
  sei();
  
  while(1)			// Endlosschleife
  {
/************************************************************/
    if(tc_rdy)		// Wenn ein neuer TC empfangen wurde :
	{
	  tc[FF_] = ((tcub[1]&0x03)*10)+(tcub[0]&0x0F);		// extrahiere Frames
	  tc[SS_] = ((tcub[3]&0x07)*10)+(tcub[2]&0x0F);		// extrahiere Sekunden
	  tc[MM_] = ((tcub[5]&0x07)*10)+(tcub[4]&0x0F);		// extrahiere Minuten
	  tc[HH_] = ((tcub[7]&0x03)*10)+(tcub[6]&0x0F);		// extrahiere Stunden
	  
	  tc[FE_] = (tcub[0]&0x0F)+48;	// extrahiere Frames Einer
	  tc[FZ_] = (tcub[1]&0x03)+48;	// extrahiere Frames Zehner
	  tc[SE_] = (tcub[2]&0x0F)+48;	// extrahiere Sekunden Einer
	  tc[SZ_] = (tcub[3]&0x07)+48;	// extrahiere Sekunden Zehner
	  tc[ME_] = (tcub[4]&0x0F)+48;	// extrahiere Minuten Einer
	  tc[MZ_] = (tcub[5]&0x07)+48;	// extrahiere Minuten Zehner
	  tc[HE_] = (tcub[6]&0x0F)+48;	// extrahiere Stunden Einer
	  tc[HZ_] = (tcub[7]&0x03)+48;	// extrahiere Stunden Zehner
	  
	  for(uint8_t i=0;i<8;i++)		// Userbits extrahieren
	  { ub[i] = tcub[i]>>4;
	  }
/*
	  usart_putc('\r');				// Cursor am Zeilenanfang positionieren

	  send_tc();					// Timecode ber USART0 senden
	  usart_putc(' ');				// Leerzeichen senden
	  send_ub();					// Userbits ber USART0 senden
*/
	  tc_rdy = 0;					// Flag zurcksetzen
	}
/************************************************************/
	// ...hier das restliche Programm...
	
  }
}

void send_tc (void)
{
  for(uint8_t i=11;i>3;i--)
  {
    usart_putc(tc[i]);			// Zehner senden
	i--;
	usart_putc(tc[i]);			// Einer senden
	if(i>4)
	{ usart_putc(':');			// Trennzeichen senden (hh:mm:ss:ff)
	}
  }
}

void send_ub (void)
{
  for(uint8_t i=7;i<8;i--)		// Userbits senden (UB8...UB1)
  {
    if(ub[i]<10)				// Werte in ASCII umwandeln und senden
	{ usart_putc(ub[i]+48);	// Werte 0 bis 9 ("0"..."9")
	}
	else
	{ usart_putc(ub[i]+55);	// Werte 10 bis 15 ("A"..."F")
	}
  }
}
